<?php

// @codingStandardsIgnoreStart
class ControllerModuleRoistat extends Controller {

    public function index() {
        $this->load->model('setting/setting');
        $settings = $this->model_setting_setting->getSetting('roistat');

        if (!array_key_exists('roistat_password', $settings) || !array_key_exists('roistat_username', $settings)) {
            echo 'Data of authorization aren\'t set';
            exit;
        }

        $action_method = $this->_getRequest('action');
        $auth_token    = $this->_getRequest('token');

        $username = $settings['roistat_username'];
        $password = $settings['roistat_password'];

        if ((string)$auth_token !== (string)md5($username . $password)) {
            echo 'Incorrect token';
            exit;
        }

        if ($action_method === 'export') {
            $output = $this->_loadOrders();
        } elseif ($action_method === 'export_clients') {
            $output = $this->_loadClients();
        } elseif ($action_method === 'lead') {
            $output = $this->_createLead();
        } else {
            $output = array('status' => 'error', 'message' => 'Action is undefined');
        }

        $this->response->addHeader('Content-Type: application/json; charset=utf8');
        $this->response->setOutput(json_encode($output));
    }

    /**
     * @return array
     */
    private function _loadOrders() {
        $result = array('orders' => array(), 'statuses' => array());

        $modifed_date = (int)$this->_getRequest('date', time() - 31*24*60*60);

        $language_id = $this->config->get('config_language_id');
        $store_id    = $this->config->get('config_store_id');

        $this->load->language('module/roistat');

        $statuses     = $this->db->query("SELECT * FROM `" . DB_PREFIX . "order_status` WHERE `language_id` = '" . $language_id . "';");
        $sql          = "SELECT * FROM `" . DB_PREFIX . "order` WHERE `store_id` = {$store_id} AND `date_modified` > {$modifed_date};";
        $orders       = $this->db->query($sql);
        $systemFields = array('order_id', 'date_added', 'order_status_id', 'total', 'roistat_visit', 'customer_id', 'shipping_method', 'payment_method', 'affiliate_id');

        foreach ($statuses->rows as $status) {
            $result['statuses'][] = array(
                'id'   => $status['order_status_id'],
                'name' => $status['name'],
            );
        }
        foreach ($orders->rows as $order) {
            $additionalFields = array();

            foreach ($order as $field => $value) {
                if (!in_array($field, $systemFields)) {
                    $additionalFields[$field] = $value;
                }
            }

            $fieldCost = array_key_exists('cost', $additionalFields) && is_scalar($additionalFields['cost'])
                ? $additionalFields['cost']
                : 0;

            $newOrder = array(
                'id'          => $order['order_id'],
                'date_create' => $order['date_added'],
                'status'      => $order['order_status_id'],
                'price'       => $order['total'],
                'cost'        => $fieldCost,
                'visit'       => $order['roistat_visit'],
                'client_id'   => $order['customer_id'],
                'fields'      => $additionalFields,
            );

            if (array_key_exists('shipping_method', $order) && $order['shipping_method'] !== '') {
                $newOrder['fields'][$this->language->get('entry_shipping_method')] = $order['shipping_method'];
            }
            if (array_key_exists('payment_method', $order) && $order['payment_method'] !== '') {
                $newOrder['fields'][$this->language->get('entry_payment_method')] = $order['payment_method'];
            }
            if (array_key_exists('affiliate_id', $order) && $order['affiliate_id'] !== '') {
                $newOrder['fields'][$this->language->get('text_affiliate')] = $order['affiliate_id'];
            }
            $result['orders'][] = $newOrder;
        }
        return $result;
    }

    /**
     * @return array
     */
    private function _createLead() {
        $customer_id = 0;
        $store_id    = $this->config->get('config_store_id');
        $title       = $this->_getRequest('title');
        $name        = $this->_getRequest('name');
        $email       = $this->_getRequest('email');
        $phone       = $this->_getRequest('phone');
        $visit       = $this->_getRequest('visit');
        $comment     = $this->_getRequest('text');
        $first_name  = ($name === '' || $name === null) ? $title : $name;

        $found_customer_id = $this->_getCustomerIdByEmail($email);
        if ($found_customer_id) {
            $customer_id = $found_customer_id;
        }

        $leadId = $this->_storeLead($title, $first_name, $email, $phone, $visit, $comment, $customer_id, $store_id);
        return array('status' => 'ok', 'order_id' => $leadId);
    }

    /**
     * @param string $email
     * @return array
     */
    private function _getCustomerIdByEmail($email) {
        $query = $this->db->query("SELECT DISTINCT * FROM " . DB_PREFIX . "customer WHERE LCASE(email) = '{$this->db->escape($email)}';");

        if (count($query->row) > 0) {
            return $query->row['customer_id'];
        } else {
            return false;
        }
    }

    /**
     * @param $title
     * @param $first_name
     * @param $email
     * @param $phone
     * @param $visit
     * @param $comment
     * @param $customer_id
     * @param $store_id
     * @return int
     */
    private function _storeLead($title, $first_name, $email, $phone, $visit, $comment, $customer_id, $store_id) {
        $store_name      = $this->config->get('config_name');
        $store_url       = $this->config->get('config_url');
        $language_id     = $this->config->get('config_language_id');
        $currency        = $this->config->get('config_currency');
        $order_status_id = $this->config->get('config_order_status_id');

        $sql = "INSERT INTO `" . DB_PREFIX . "order`
                        SET `order_status_id` = '{$order_status_id}',
                            `language_id` = '{$language_id}',
                            `telephone` = '{$this->db->escape($phone)}',
                            `email` = '{$this->db->escape($email)}',
                            `firstname` = '{$this->db->escape($first_name)}',
                            `customer_id` = '{$customer_id}',
                            `store_url` = '{$store_url}',
                            `store_name` = '{$this->db->escape($title)} ({$store_name})',
                            `store_id` = '{$store_id}',
                            `comment` = '{$this->db->escape($comment)}',
                            `roistat_visit` = '{$this->db->escape($visit)}',
                            `payment_code` = 'cod',
                            `currency_code` = '{$currency}',
                            `date_added` = NOW(),
                            `date_modified` = NOW()";
        $this->db->query($sql);
        return $this->db->getLastId();
    }

    /**
     * @return array
     */
    private function _loadClients() {
        return array();
    }

    /**
     * @param string $key
     * @param mixed $default
     * @return string
     */
    private function _getRequest($key, $default = null) {
        return array_key_exists($key, $this->request->request) ? $this->request->request[$key] : $default;
    }

    /**
     * For AJAX in order_form
     */
    public function updateVisit() {
        $this->load->language('module/roistat');

        if ($this->_isAdmin()) {
            $order_id = $this->_getRequest('order_id');
            $visit_id = $this->_getRequest('visit');
            $this->_updateVisitByOrderId($order_id, $visit_id);

            $output = array('success' => $this->language->get('text_edit_visit_success'));
        } else {
            $output = array('error' => $this->language->get('text_edit_visit_error'));
        }
        $this->response->addHeader('Content-Type: application/json; charset=utf8');
        $this->response->setOutput(json_encode($output));
    }

    /**
     * @param int $order_id
     * @param string $visit
     */
    private function _updateVisitByOrderId($order_id, $visit) {
        $this->db->query("UPDATE `" . DB_PREFIX . "order` SET `roistat_visit` = '" . $visit . "' WHERE `order_id` = '" . (int)$order_id . "';");
    }

    /**
     * @return bool
     */
    private function _isAdmin() {
        $user = new User($this->registry);
        return (bool)$user->isLogged();
    }
}
// @codingStandardsIgnoreEnd